/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include <vcl.h>
#include <process.h>
#pragma hdrstop

#include "Main.h"
#include "About.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    ::CoInitialize(NULL);
    ::WSAStartup(MAKEWORD(2, 0), &wsaData);
    cap = NULL;

    Application->Title = "Client - " + Caption;

    ZeroMemory(&wfx, sizeof(wfx));
    wfx.wFormatTag = WAVE_FORMAT_PCM;
    wfx.nChannels = 2;
    wfx.nSamplesPerSec = 44100;
    wfx.wBitsPerSample = 16;
    wfx.nBlockAlign = 4;
    wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
    wfx2 = wfx;

    DoubleBuffered = true;
    Devices();
    AudioOutList();

    R0->DoubleBuffered = true;
    R1->DoubleBuffered = true;
    R2->DoubleBuffered = true;
    R3->DoubleBuffered = true;
    R4->DoubleBuffered = true;
    R5->DoubleBuffered = true;
    R6->DoubleBuffered = true;
    R7->DoubleBuffered = true;
    R8->DoubleBuffered = true;
    R9->DoubleBuffered = true;
    R10->DoubleBuffered = true;
    R11->DoubleBuffered = true;
    R12->DoubleBuffered = true;
    R13->DoubleBuffered = true;
    R14->DoubleBuffered = true;
    R15->DoubleBuffered = true;

    hWnd[0] = R0->Handle;
    hWnd[1] = R1->Handle;
    hWnd[2] = R2->Handle;
    hWnd[3] = R3->Handle;
    hWnd[4] = R4->Handle;
    hWnd[5] = R5->Handle;
    hWnd[6] = R6->Handle;
    hWnd[7] = R7->Handle;
    hWnd[8] = R8->Handle;
    hWnd[9] = R9->Handle;
    hWnd[10] = R10->Handle;
    hWnd[11] = R11->Handle;
    hWnd[12] = R12->Handle;
    hWnd[13] = R13->Handle;
    hWnd[14] = R14->Handle;
    hWnd[15] = R15->Handle;
    
    LoadParams();
    // set buttons status
    OnChange(0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnStart(TObject *Sender)
{
    StartBtn->ImageIndex = (StartBtn->Down) ? 1 : 0;
    if(StartBtn->Down)
    {
        /*
        Host->Enabled = false;
        Channel->Enabled = false;
        Port->Enabled = false;
        AudioSources->Enabled = false;
        VideoSources->Enabled = false;
        AudioOut->Enabled = false;
        */
        StartUp();
    }
    else
    {
        CleanUp();
        /*
        Host->Enabled = true;
        Channel->Enabled = true;
        Port->Enabled = true;
        AudioSources->Enabled = true;
        VideoSources->Enabled = true;
        AudioOut->Enabled = true;
        */
        InvRenderers();
        PttBtn->Down = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartUp()
{
    StartBtn->Enabled = false;

    InitializeContext();
    video.Open(cntx);
    audio.Open(cntx);
    cntx.video_rend = &video;
    cntx.audio_rend = &audio;

    cntx.net = new CNetwork(cntx);
    cap = new CCapture();
    if(cap)
    {
        cntx.v_call = new CVideoIn(cntx);
        cntx.a_call = new CAudioIn(cntx);
        cap->InitGraph(cntx);
        cap->Start();
    }

    StartBtn->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CleanUp()
{
    StartBtn->Enabled = false;

    if(cap)
    {
        cap->Stop();
        delete cap;
    }

    if(cntx.v_call)
    {
        delete cntx.v_call;
    }

    if(cntx.a_call)
    {
        delete cntx.a_call;
    }

    if(cntx.net)
    {
        delete cntx.net;
    }

    if(cntx.err)
    {
        delete cntx.err;
        cntx.err = NULL;
    }

    // clean context
    memset(&cntx, 0, sizeof(cntx));
    audio.Close();
    video.Close();

    StartBtn->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadParams()
{
    Load();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveParams()
{
    Save();
}
//---------------------------------------------------------------------------
void TMainForm::Save()
{
    TRegistry *reg = new TRegistry();
    try
    {
        reg->Access = KEY_ALL_ACCESS;
        reg->RootKey = HKEY_CURRENT_USER;
        reg->OpenKey("\\SOFTWARE\\Gmax\\Vic", true);
        reg->WriteInteger("Video", VideoSources->ItemIndex);
        reg->WriteInteger("Audio", AudioSources->ItemIndex);
        reg->WriteString("Server", Host->Text);
        reg->WriteInteger("Channel", Channel->Position);
        reg->CloseKey();
    }
    catch(...)
    {
    }
    delete reg;
 }
//---------------------------------------------------------------------------
void TMainForm::Load()
{
    TRegistry *reg = new TRegistry();
    try
    {
        reg->Access = KEY_ALL_ACCESS;
        reg->RootKey = HKEY_CURRENT_USER;
        reg->OpenKey("\\SOFTWARE\\Gmax\\Vic", true);
        VideoSources->ItemIndex = reg->ReadInteger("Video");
        AudioSources->ItemIndex = reg->ReadInteger("Audio");
        Host->Text = reg->ReadString("Server");
        Channel->Position = reg->ReadInteger("Channel");
        reg->CloseKey();
    }
    catch(...)
    {
    }
    delete reg;
 }
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeContext()
{
    // protectie
    memset(&cntx, 0, sizeof(cntx));
    cntx.ch_inp = Channel->Position;
    // net sink
    strcpy(cntx.host, GetHost());
    cntx.port = GetPort();
    cntx.ch_out = Channel->Position;
    cntx.on_air = PttBtn->Down;

    cntx.is_dsvideo = GetVideoIndex();
    wcscpy(cntx.video, GetVideoSource());
    cntx.is_dsaudio = GetAudioIndex();
    wcscpy(cntx.audio, GetAudioSource());

    cntx.preview = 0;
    cntx.hWndRend = &hWnd[0];
    cntx.hWndPrev = NULL;
    cntx.is_video = isVideo();
    cntx.is_audio = isAudio();
    cntx.master = GetMasterStream();
    cntx.abitrate = GetAudioDataRate();
    cntx.vbitrate = GetVideoDataRate();

    cntx.rate = 10; //GetFrameRate() ? 15 : 25;
    cntx.key_rate = GetKeyFrameRate();
    POINT p = GetFrameSize();
    cntx.width = p.x;
    cntx.height = p.y;
    RECT r = GetRendererSize();
    cntx.x = r.left;
    cntx.y = r.top;
    cntx.dx = r.right;
    cntx.dy = r.bottom;
//    cntx.quality = 1;
    cntx.wfx = &wfx;
    cntx.wfx2 = &wfx2;
    cntx.sound = AudioOut->ItemIndex;
    // err
    cntx.err = new CErrorHandler(Handle);
    cntx.acodec_id = 1;
    cntx.vcodec_id = 1;
    // DV
    cntx.dv_res = GetDVFormat();
    cntx.half_rate = GetFrameRate();
    // Vu metru
    cntx.hWndWu = Handle;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::VideoBtnClick(TObject *Sender)
{
    if(cap)
    {
        cap->VideoProperty(Handle);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AudioBtnClick(TObject *Sender)
{
    if(cap)
    {
        cap->AudioProperty(Handle);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnFormClose(TObject *Sender, TCloseAction &Action)
{
    int nResult = ::MessageBox(Handle,
        _T("Are you sure?"),
        _T("Exit"),
        MB_ICONQUESTION | MB_YESNO);
    if(nResult == IDNO)
    {
        Action = caNone;
    }
    else
    {
        CleanUp();
        nResult = ::MessageBox(Handle,
            _T("Do you want to save the settings?"),
            _T("Save"),
            MB_ICONQUESTION | MB_YESNO);
        if(nResult == IDYES)
        {
            SaveParams();
        }
        ::WSACleanup();
        ::CoUninitialize();
        AboutForm->ShowModal();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Devices()
{
    VideoSources->Items->Clear();
    VideoSources->Items->Add("(none)");
    AudioSources->Items->Clear();
    AudioSources->Items->Add("(none)");
    CDevice::GetSources(VideoSources->Handle, 0);
    CDevice::GetSources(AudioSources->Handle, 1);
    VideoSources->ItemIndex = 0;
    AudioSources->ItemIndex = 0;
}
//---------------------------------------------------------------------------
LPWSTR TMainForm::GetVideoSource()
{
    wchar_t *tmp = WideString(VideoSources->Text).c_bstr();
    return (tmp == NULL) ? L"?" : tmp;
}
//---------------------------------------------------------------------------
LPWSTR TMainForm::GetAudioSource()
{
    wchar_t *tmp = WideString(AudioSources->Text).c_bstr();
    return (tmp == NULL) ? L"?" : tmp;
}
//---------------------------------------------------------------------------
bool TMainForm::isVideo()
{
    return VideoSources->ItemIndex;
}
//---------------------------------------------------------------------------
bool TMainForm::isAudio()
{
    return AudioSources->ItemIndex;;
}
//---------------------------------------------------------------------------
int TMainForm::GetVideoIndex()
{
    return VideoSources->ItemIndex;
}
//---------------------------------------------------------------------------
int TMainForm::GetAudioIndex()
{
    return AudioSources->ItemIndex;
}
//---------------------------------------------------------------------------
int TMainForm::GetMasterStream()
{
    return 1;
}
//---------------------------------------------------------------------------
LPSTR TMainForm::GetHost()
{
    return Host->Text.c_str();
}
//---------------------------------------------------------------------------
int TMainForm::GetPort()
{
    return 1958;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
POINT TMainForm::GetFrameSize()
{
    p.x = 320;
    p.y = 240;
    return p;
}
//---------------------------------------------------------------------------
RECT TMainForm::GetRendererSize()
{
    r.left = 1;
    r.top = 1;
    r.right = 160;
    r.bottom = 120;
    return r;
}
//---------------------------------------------------------------------------
int TMainForm::GetFrameRate()
{
    return 0;
}
//---------------------------------------------------------------------------
int TMainForm::GetVideoDataRate()
{
    return 64000;
}
//---------------------------------------------------------------------------
int TMainForm::GetAudioDataRate()
{
    return 16000;
}
//---------------------------------------------------------------------------
int TMainForm::GetKeyFrameRate()
{
    return 16;
}
//---------------------------------------------------------------------------
int TMainForm::GetDVFormat()
{
    return 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnChange(TObject *Sender)
{
    // set buttons status
    VideoBtn->Enabled = isVideo();
    AudioBtn->Enabled = isAudio();
    StartBtn->Enabled = (isVideo() || isAudio());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AudioOutList()
{
    WAVEOUTCAPS caps;
    DWORD n = ::waveOutGetNumDevs();
    SndBtn->Enabled = n;

    AudioOut->Items->Clear();
    for(unsigned i = 0;i < n;i++)
    {
        ::waveOutGetDevCaps(i, &caps, sizeof(caps));
        AudioOut->Items->Add(caps.szPname);
    }
    AudioOut->ItemIndex = 0;

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnChannel(TObject *Sender)
{
    TSpeedButton *btn = (TSpeedButton *)Sender;
    cntx.ch_inp = (btn->Down) ? btn->Tag : 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PttClick(TObject *Sender)
{
    cntx.on_air = PttBtn->Down;
    PttBtn->ImageIndex = (PttBtn->Down) ? 6 : 5;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if(cntx.connected && Key == VK_ESCAPE)
    {
        PttBtn->Down = !PttBtn->Down;
        PttClick(Sender);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SndBtnClick(TObject *Sender)
{
    ::WinExec("control.exe mmsys.cpl", SW_SHOW);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InvRenderers()
{
    R0->Invalidate();
    R1->Invalidate();
    R2->Invalidate();
    R3->Invalidate();
    R4->Invalidate();
    R5->Invalidate();
    R6->Invalidate();
    R7->Invalidate();
    R8->Invalidate();
    R9->Invalidate();
    R10->Invalidate();
    R11->Invalidate();
    R12->Invalidate();
    R13->Invalidate();
    R14->Invalidate();
    R15->Invalidate();
}
//---------------------------------------------------------------------------


