/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#ifndef MainH
#define MainH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Dialogs.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <Menus.hpp>
#include <ExtCtrls.hpp>
#include <jpeg.hpp>
#include <Graphics.hpp>
#include <XPMan.hpp>
#include <ImgList.hpp>
#include <Buttons.hpp>
#include "Registry.hpp"
#include "Context.h"
#include "ErrorHandler.h"
#include "Network.h";
#include "Device.h";
#include "Capture.h";
#include "VideoIn.h";
#include "AudioIn.h";
#include "AudioRenderer.h";
#include "VideoRenderer.h";
//---------------------------------------------------------------------------

class TMainForm : public TForm
{
__published: // IDE-managed Components
    TXPManifest *XPManifest1;
    TToolBar *ToolBar1;
    TEdit *Host;
    TEdit *Port;
    TUpDown *Channel;
    TComboBox *VideoSources;
    TComboBox *AudioSources;
    TComboBox *AudioOut;
    TToolButton *StartBtn;
    TImageList *ImageList1;
    TToolButton *VideoBtn;
    TToolButton *AudioBtn;
    TToolButton *SndBtn;
    TToolButton *ToolButton1;
    TToolButton *ToolButton2;
    TToolButton *ToolButton3;
    TToolButton *ToolButton4;
    TPanel *Panel1;
    TPanel *R0;
    TPanel *R1;
    TPanel *R2;
    TPanel *R3;
    TPanel *R7;
    TPanel *R6;
    TPanel *R5;
    TPanel *R4;
    TPanel *R8;
    TPanel *R9;
    TPanel *R10;
    TPanel *R11;
    TPanel *R15;
    TPanel *R14;
    TPanel *R13;
    TPanel *R12;
    TToolButton *ToolButton5;
    TToolButton *PttBtn;
    TToolButton *ToolButton6;
    TToolButton *ToolButton7;
    void __fastcall OnStart(TObject *Sender);
    void __fastcall ExitClick(TObject *Sender);
    void __fastcall VideoBtnClick(TObject *Sender);
    void __fastcall AudioBtnClick(TObject *Sender);
    void __fastcall OnFormClose(TObject *Sender, TCloseAction &Action);
    void __fastcall OnChange(TObject *Sender);
    void __fastcall OnChannel(TObject *Sender);
    void __fastcall PttClick(TObject *Sender);
    void __fastcall FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
    void __fastcall SndBtnClick(TObject *Sender);
private:
    // streams
    void __fastcall Devices();
    LPWSTR GetVideoSource();
    LPWSTR GetAudioSource();
    bool isVideo();
    bool isAudio();
    int GetVideoIndex();
    int GetAudioIndex();
    int GetMasterStream();
    LPSTR GetHost();
    int GetPort();

    // video and audio
    POINT p;
    RECT r;

    POINT GetFrameSize();
    RECT GetRendererSize();
    int GetFrameRate();
    int GetVideoDataRate();
    int GetAudioDataRate();
    int GetKeyFrameRate();
    int GetDVFormat();

    // logo
    void __fastcall DrawImage();
    int __fastcall GetPosition();
    void __fastcall SetPosition(int pos);
    void __fastcall InvRenderers();

    void __fastcall AudioOutList();
    void __fastcall InitializeContext();
    void Load();
    void Save();

    WSADATA wsaData;
    GLOBAL_CONTEXT cntx;
    WAVEFORMATEX wfx;
    WAVEFORMATEX wfx2;
    CAudioRenderer audio;
    CVideoRenderer video;
    CCapture *cap;
    HWND hWnd[16];
public: // User declarations
    __fastcall TMainForm(TComponent* Owner);
    void __fastcall LoadParams();
    void __fastcall SaveParams();
    void __fastcall StartUp();
    void __fastcall CleanUp();
};

//---------------------------------------------------------------------------
extern PACKAGE TMainForm *MainForm;
//---------------------------------------------------------------------------
#endif

