/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "VideoDecoder.h"

// ctor
CVideoDecoder::CVideoDecoder()
{
    got_picture = 0;
    // init
    ctx = NULL;
    codec = NULL;
    pict = NULL;
    // prepare
    __avcodec_init();
    __avcodec_register_all();
}
//---------------------------------------------------------------------------
// dtor
CVideoDecoder::~CVideoDecoder()
{
}
//---------------------------------------------------------------------------
// video
int CVideoDecoder::open(GLOBAL_CONTEXT &oa_ctx)
{
    codec = (AVCodec *)__avcodec_find_decoder(CODEC_ID_H264);
    if(!codec)
    {
        return 0;
    }

    ctx = (AVCodecContext *)__avcodec_alloc_context();
    if(!ctx)
    {
        return 0;
    }

    ctx->bit_rate = oa_ctx.vbitrate;
    ctx->time_base.num = 1;
    ctx->time_base.den = oa_ctx.rate;
    ctx->width = oa_ctx.width;
    ctx->height = oa_ctx.height;
    ctx->pix_fmt = PIX_FMT_YUV420P;
    ctx->gop_size = oa_ctx.key_rate;

    // initialize context
    ctx->width = oa_ctx.width;
    ctx->height = oa_ctx.height;
    ctx->pix_fmt = PIX_FMT_YUV420P;

    if(__avcodec_open(ctx, codec) < 0)
    {
        return 0;
    }

    // alloc picture frame
    pict = (AVFrame *)__avcodec_alloc_frame();
    return (pict != NULL);
}
//---------------------------------------------------------------------------
int CVideoDecoder::decode(uint8_t *inp, int inp_size, uint8_t *out, int key_frame)
{
    got_picture = 0;

    // prepare image
    pict->key_frame = key_frame;
    if(__avcodec_decode_video(ctx, pict, &got_picture, inp, inp_size) < 0)
    {
        return 0;
    }

    if(got_picture)
    {
        int off = get(0, out, ctx->width, ctx->height);
        off += get(1, out+off, ctx->width/2, ctx->height/2);
        off += get(2, out+off, ctx->width/2, ctx->height/2);
    }

    return got_picture;
}
//---------------------------------------------------------------------------
int CVideoDecoder::get(int x, uint8_t *dst, int w, int h)
{
    uint8_t *src = pict->data[x];
    for(int i = 0;i < h; i++)
    {
        memmove(dst, src, w);
        src += pict->linesize[x];
        dst += w;
    }
    return (w * h);
}
//---------------------------------------------------------------------------
void CVideoDecoder::close()
{
    if(ctx != NULL)
    {
        __avcodec_close(ctx);
        __av_free(ctx);
        ctx = NULL;
    }

    if(pict != NULL)
    {
        __av_free(pict);
        pict = NULL;
    }
}
//---------------------------------------------------------------------------

