/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "VideoEncoder.h"

//---------------------------------------------------------------------------
// ctor
CVideoEncoder::CVideoEncoder()
{
    // init
    ctx = NULL;
    codec = NULL;
    pict = NULL;
    // prepare
    __avcodec_init();
    __avcodec_register_all();
}
//---------------------------------------------------------------------------
// dtor
CVideoEncoder::~CVideoEncoder()
{
}
//---------------------------------------------------------------------------
// video
int CVideoEncoder::open(GLOBAL_CONTEXT &cntx)
{
    codec = (AVCodec *)__avcodec_find_encoder(CODEC_ID_H264);
    if(!codec)
    {
        return 0;
    }

    ctx = (AVCodecContext *)__avcodec_alloc_context();
    if(!ctx)
    {
        return 0;
    }

    // initialize context
    ctx->bit_rate = cntx.vbitrate;
    ctx->time_base.num = 1;
    ctx->time_base.den = cntx.rate;
    ctx->width = cntx.width;
    ctx->height = cntx.height;
    ctx->pix_fmt = PIX_FMT_YUV420P;
    ctx->gop_size = cntx.key_rate;
    ctx->me_method = 8;
    ctx->noise_reduction = 100;

    if(__avcodec_open(ctx, codec) < 0)
    {
        return 0;
    }

    // alloc picture frame
    pict = (AVFrame *)__avcodec_alloc_frame();
    if(pict == NULL)
    {
        return 0;
    }

    int size = ctx->width * ctx->height;
    pict->data[0] = (uint8_t *)malloc(size * 3 / 2);
    pict->data[1] = pict->data[0] + size;
    pict->data[2] = pict->data[1] + size / 4;
    pict->linesize[0] = ctx->width;
    pict->linesize[1] = ctx->width / 2;
    pict->linesize[2] = ctx->width / 2;
    pict->quality = cntx.quality;

    return 1;
}
//---------------------------------------------------------------------------
int CVideoEncoder::encode(RGBTRIPLE *inp, uint8_t *out, int inp_size, int *key_frame)
{
    RGB2I420P(inp, TRUE);
    int frame_size = __avcodec_encode_video(ctx, out, inp_size, pict);
    *key_frame = ctx->coded_frame->key_frame;
    return frame_size;
}
//---------------------------------------------------------------------------
void CVideoEncoder::RGB2I420P(RGBTRIPLE *rgb, uint8_t flip)
{
    // get pointers to the data
    RGBTRIPLE *tmp = rgb;
    uint8_t *yline = pict->data[0];
    uint8_t *uline = pict->data[1];
    uint8_t *vline = pict->data[2];
    // convert
    for(int32_t y = 0;y < ctx->height; y++)
    {
        // flip image ?
        if(flip)
        {
            tmp = rgb + ctx->width * (ctx->height - 1 - y);
        }
        // encode I420
        for(int32_t x = 0;x < ctx->width;x++)
        {
            *yline = Y(tmp->rgbtRed, tmp->rgbtGreen, tmp->rgbtBlue);
            if(!((x | y) & 0x01))
            {
                *uline = U(tmp->rgbtBlue, *yline);
                *vline = V(tmp->rgbtRed, *yline);
                uline++;
                vline++;
            }
            yline++;
            tmp++;
        }
    }
}
//---------------------------------------------------------------------------
void CVideoEncoder::close()
{
    if(ctx != NULL)
    {
        __avcodec_close(ctx);
        __av_free(ctx);
        ctx = NULL;
    }

    if(pict->data[0] != NULL)
    {
        free(pict->data[0]);
        pict->data[0] = 0;
    }

    if(pict != NULL)
    {
        __av_free(pict);
        pict = NULL;
    }
}
//---------------------------------------------------------------------------

