/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "Network.h"

//---------------------------------------------------------------------------
CNetwork::CNetwork(GLOBAL_CONTEXT &p_cntx) : CMonitor()
{
    cntx = &p_cntx;

    // open encoders
    vid.open(p_cntx);
    chunks = 5;
    frame_size = aud.open(p_cntx);
    chunk_size = frame_size * cntx->wfx->nBlockAlign;
    cntx->frame_size = chunks * chunk_size;

    // network
    sink = new CPushSink(p_cntx);
    if(sink)
    {
        sink->OpenSink();
    }
}
//---------------------------------------------------------------------------
CNetwork::~CNetwork()
{
    // close network sink
    if(sink)
    {
        sink->CloseSink();
        delete sink;
        sink = NULL;
    }
    // close compressors
    vid.close();
    aud.close();
}
//---------------------------------------------------------------------------
void CNetwork::OnVideo(uint8_t *buffer, uint32_t size)
{
    if(sink)
    {
        if((cntx->master) ? TryLock() : Lock())
        {
            // send
            EncodeVideo(buffer, size);
            Unlock();
        }
    }
}
//---------------------------------------------------------------------------
void CNetwork::EncodeVideo(uint8_t *buffer, uint32_t size)
{
    uint8_t *out = (uint8_t *)malloc(size);
    if(out)
    {
        int32_t key = 0;
        int32_t out_size = vid.encode((RGBTRIPLE *)buffer, out, size, &key);
        if(out_size > 0)
        {
            sink->SendVideo(out, out_size, key);
        }
        free(out);
    }
}
//---------------------------------------------------------------------------
void CNetwork::OnAudio(uint8_t *buffer, uint32_t size)
{
    if(sink)
    {
        Lock();
        // send
        EncodeAudio(buffer, size);
        Unlock();
    }
}
//---------------------------------------------------------------------------
void CNetwork::EncodeAudio(uint8_t *buffer, uint32_t size)
{
    // encode
    uint8_t *out = (uint8_t *)malloc(size);
    int32_t out_size = 0;
    if(out)
    {
        // encode
        for(uint32_t i = 0;i < chunks;i++)
        {
            out_size += aud.encode(out + out_size, frame_size, (short*)buffer);
            buffer += chunk_size;
        }
        // send
        if(out_size > 0)
        {
            sink->SendAudio(out, out_size);
        }
        // clean up
        free(out);
    }
}
//---------------------------------------------------------------------------
void CNetwork::ReOpenVideo()
{
    Lock();
    vid.close();
    vid.open(*cntx);
    Unlock();
}
//---------------------------------------------------------------------------
void CNetwork::ReOpenAudio()
{
    Lock();
    aud.close();
    frame_size = aud.open(*cntx);
    chunk_size = frame_size * cntx->wfx->nBlockAlign;
    cntx->frame_size = chunks * chunk_size;
    Unlock();
}
//---------------------------------------------------------------------------

