/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "PushSink.h"

//---------------------------------------------------------------------------
// ctor
CPushSink::CPushSink(GLOBAL_CONTEXT &p_cntx)
{
    cntx = &p_cntx;
    cli = INVALID_SOCKET;
    down = NULL;
}
//---------------------------------------------------------------------------
// dtor
CPushSink::~CPushSink()
{
}
//---------------------------------------------------------------------------
void CPushSink::OpenSink()
{
    cli = ::socket(AF_INET, SOCK_STREAM, 0);
    if(cli != INVALID_SOCKET)
    {
        ss.sin_family = AF_INET;
        ss.sin_port = htons(cntx->port);
        ss.sin_addr.s_addr = host_addr(cntx->host);
        cntx->connected = (::connect(cli, (sockaddr *)&ss, sizeof(ss)) == 0);
        if(!cntx->connected)
        {
            Error(SYS_ERROR, ::WSAGetLastError());
            CloseSink();
        }
        else
        {
            cntx->cli = cli;
            down = new CDownStream(cntx);
            if(down)
            {
                down->StartThread();
            }
        }
    }
    else
    {
        Error(SYS_ERROR, ::WSAGetLastError());
    }
}
//---------------------------------------------------------------------------
void CPushSink::CloseSink()
{
    ::shutdown(cli, SD_BOTH);
    ::closesocket(cli);
    cli = INVALID_SOCKET;
    cntx->cli = INVALID_SOCKET;
    cntx->connected = 0;

    if(down)
    {
        down->StopThread();
        down->WaitFor();
        delete down;
        down = NULL;
    }
}
//---------------------------------------------------------------------------
int CPushSink::SendVideo(uint8_t *buffer, uint32_t size, uint32_t key)
{
    if(!cntx->connected)
    {
        return 1;
    }

    /*
    if(!cntx->on_air)
    {
        return 1;
    }
    */

    // header
    hdr_vid.payload = ID_VIDEO;
    hdr_vid.ch = cntx->ch_out;
    hdr_vid.magic = MAGIC;
    hdr_vid.key = key;
    hdr_vid.w = cntx->width;
    hdr_vid.h = cntx->height;
    hdr_vid.size = size;

    if(::send(cli, (int8_t *)&hdr_vid, sizeof(hdr_vid), 0) == INVALID_SOCKET)
    {
        Error(SYS_ERROR, ::WSAGetLastError());
        CloseSink();
    }

    if(::send(cli, buffer, size, 0) == INVALID_SOCKET)
    {
        Error(SYS_ERROR, ::WSAGetLastError());
        CloseSink();
    }

    return 0;
}
//---------------------------------------------------------------------------
int CPushSink::SendAudio(uint8_t *buffer, uint32_t size)
{
    if(!cntx->connected)
    {
        return 1;
    }

    if(!cntx->on_air)
    {
        return 1;
    }

    // header
    hdr_aud.payload = ID_AUDIO;
    hdr_aud.ch = cntx->ch_out;
    hdr_aud.magic = MAGIC;
    hdr_aud.w = cntx->wfx->nSamplesPerSec;
    hdr_aud.h = cntx->wfx->nChannels;
    hdr_aud.key = 0;
    hdr_aud.size = size;

    if(::send(cli, (int8_t *)&hdr_aud, sizeof(hdr_aud), 0) == INVALID_SOCKET)
    {
        Error(SYS_ERROR, ::WSAGetLastError());
        CloseSink();
    }

    if(::send(cli, buffer, size, 0) == INVALID_SOCKET)
    {
        Error(SYS_ERROR, ::WSAGetLastError());
        CloseSink();
    }

    return 0;
}
//---------------------------------------------------------------------------
void CPushSink::Error(DWORD wParam, DWORD lParam)
{
    if(cntx->err)
    {
        cntx->err->Error(wParam, lParam);
    }
}
//---------------------------------------------------------------------------
unsigned long CPushSink::host_addr(LPCSTR host)
{
    unsigned long ip = inet_addr(host);

    if(ip != INADDR_NONE) // invalid IP
    {
        return ip;
    }

    // by name
    hostent *hent = ::gethostbyname(host);
    if(!hent)
    {
        return INADDR_NONE;
    }

    LPBYTE s = hent->h_addr_list[0];
    if(!s)
    {
        return INADDR_NONE;
    }

    ip = (s[3] << 24) | (s[2] << 16) | (s[1] << 8) | s[0];
    return ip;
}
//---------------------------------------------------------------------------

