/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "AudioRenderer.h"

//---------------------------------------------------------------------------
//ctor
CAudioRenderer::CAudioRenderer()
{
    ii = 0;
}
//---------------------------------------------------------------------------
//dtor
CAudioRenderer::~CAudioRenderer()
{
}
//---------------------------------------------------------------------------
void CAudioRenderer::Open(GLOBAL_CONTEXT &ctx)
{
    actx = &ctx;
    dec.open(ctx);
    ::waveOutOpen(&hwo, actx->sound, actx->wfx2, 0, 0, CALLBACK_NULL|WAVE_MAPPED);
    for(int i = 0;i < RINGSIZE;i++)
    {
        memset(&whdr[i], 0, sizeof(WAVEHDR));
        whdr[i].dwBufferLength = AVCODEC_MAX_AUDIO_FRAME_SIZE;
        whdr[i].lpData = new char[whdr[i].dwBufferLength];
        ::waveOutPrepareHeader(hwo, &whdr[i], sizeof(WAVEHDR));
    }
}
//---------------------------------------------------------------------------
void CAudioRenderer::Close()
{
    // close
    ::waveOutReset(hwo);
    ::waveOutClose(hwo);
    // clean up
    for(int i = 0;i < RINGSIZE;i++)
    {
        if(whdr[i].lpData != NULL)
        {
            delete whdr[i].lpData;
            whdr[i].lpData = NULL;
        }
    }
    // clean up
    dec.close();
}
//---------------------------------------------------------------------------
int CAudioRenderer::Decode(unsigned char *tmp_out, unsigned char *tmp_inp, int inp_size)
{
    int out_size = 0;
    while(inp_size > 0)
    {
        int tmp_out_size = 0;
        int len = dec.decode((short *)tmp_out,
            &tmp_out_size, tmp_inp, inp_size);
        if(len < 0)
        {
            // error
            return len;
        }
        tmp_out  += tmp_out_size;
        out_size += tmp_out_size;
        tmp_inp  += len;
        inp_size -= len;
    }
    return out_size;
}
//---------------------------------------------------------------------------
int CAudioRenderer::Play(unsigned char *data, int size)
{
    int out_size = Decode(out, data, size);
    if((out_size > 0) && (out_size <= AVCODEC_MAX_AUDIO_FRAME_SIZE))
    {
        whdr[ii].dwBufferLength = out_size;
        memmove(whdr[ii].lpData, out, whdr[ii].dwBufferLength);
        ::waveOutWrite(hwo, &whdr[ii], sizeof(WAVEHDR));
        ii = (ii + 1) % RINGSIZE;
    }
    return 0;
}
//---------------------------------------------------------------------------
DWORD CAudioRenderer::GetVolume()
{
    DWORD Volume;
    ::waveOutGetVolume(hwo, &Volume);
    return Volume;
}
//---------------------------------------------------------------------------
void CAudioRenderer::SetVolume(DWORD Volume)
{
    ::waveOutSetVolume(hwo, Volume);
}
//---------------------------------------------------------------------------

