/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/
#include "DownStream.h"

//---------------------------------------------------------------------------
CDownStream::CDownStream(GLOBAL_CONTEXT *c) : CThread()
{
    // init
    cntx = c;
    block_body = NULL;
}
//---------------------------------------------------------------------------
CDownStream::~CDownStream()
{
    if(block_body)
    {
        free(block_body);
        block_body = NULL;
    }
}
//---------------------------------------------------------------------------
void CDownStream::Run()
{
    while(!Terminated())
    {
        // read block
        if(ReadBlock() && hdr.magic == CIGAM)
        {
            switch(hdr.payload)
            {
                // video
                case ID_VIDEO:
                    if(cntx->video_rend)
                    {
                        cntx->video_rend->DrawFrame(block_body, block_size, hdr.key, hdr.ch);
                    }
                    break;
                // audio
                case ID_AUDIO:
                    if(cntx->audio_rend)
                    {
                        cntx->audio_rend->Play(block_body, block_size);
                    }
                    break;
                default:
                    break;
            }
        }
        else
        {
            ::Sleep(50);
        }
    }
}
//---------------------------------------------------------------------------
int32_t CDownStream::ReadBlock()
{
    // read the block header
    if(Read((uint8_t *)&hdr, sizeof(hdr)) != sizeof(hdr))
    {
        return 0;
    }
    // compute the size of the block
    block_size = hdr.size;
    // alloc block
    if(block_body)
    {
        free(block_body);
    }
    block_body = (uint8_t *)malloc(block_size);
    if(!block_body)
    {
        return 0;
    }
    // read the block
    return (Read(block_body, block_size) == block_size);
}
//---------------------------------------------------------------------------
void CDownStream::ShowError()
{
    if(cntx->err)
    {
        cntx->err->Error(SYS_ERROR, ::WSAGetLastError());
    }
}
//---------------------------------------------------------------------------
int32_t CDownStream::Read(uint8_t *buffer, int32_t size)
{
    int32_t inp = size;
    uint8_t *tmp = buffer;
    while(inp)
    {
        int r = ::recv(cntx->cli, tmp, inp, 0);
        if(!r || (r == SOCKET_ERROR))
        {
            // error
            return 0;
        }
        // next read
        inp -= r;
        tmp += r;
    }
    return size;
}
//---------------------------------------------------------------------------

