/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include "Client.h"
#include "Gateway.h"

//---------------------------------------------------------------------------
CClient::CClient(SRV_CONTEXT *c) : CThread()
{
    // init
    cntx = c;
    cli = cntx->cli;
    block_body = NULL;
}
//---------------------------------------------------------------------------
CClient::~CClient()
{
    if(block_body)
    {
        free(block_body);
        block_body = NULL;
    }
}
//---------------------------------------------------------------------------
void CClient::Close()
{
    if(cli != INVALID_SOCKET)
    {
        ::shutdown(cli, SD_BOTH);
        ::closesocket(cli);
    }
}
//---------------------------------------------------------------------------
void CClient::Run()
{
    while(!Terminated())
    {
        if(cntx->gate)
        {
            // read block
            if(!ReadBlock())
            {
                break;
            }
            // send to all
            cntx->gate->Send(cli, (uint8_t *)&hdr, sizeof(hdr),
                block_body, block_size);
        }
        else
        {
            ::Sleep(10);
        }
    }
}
//---------------------------------------------------------------------------
int32_t CClient::ReadBlock()
{
    // read the block header
    if(Read((uint8_t *)&hdr, sizeof(hdr)) != sizeof(hdr))
    {
        return 0;
    }
    // invert
    if(hdr.magic != MAGIC)
    {
        return 0;
    }
    hdr.magic = CIGAM;
    // compute the size of the block
    block_size = hdr.size;
    // alloc block
    if(block_body)
    {
        free(block_body);
    }
    block_body = (uint8_t *)malloc(block_size);
    if(!block_body)
    {
        return 0;
    }
    // read the block
    return (Read(block_body, block_size) == block_size);
}
//---------------------------------------------------------------------------
int32_t CClient::Read(uint8_t *buffer, int32_t size)
{
    int32_t inp = size;
    uint8_t *tmp = buffer;
    while(inp > 0)
    {
        int r = ::recv(cli, tmp, inp, 0);
        if((r == 0) || (r == SOCKET_ERROR))
        {
            return 0;
        }
        // next read
        inp -= r;
        tmp += r;
    }
    return size;
}
//---------------------------------------------------------------------------

