/****************************************************************************
* Together - Simple Videoconference System
* Copyright (C) 2010 Marius Gligor info@gmax.ws
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include "Gateway.h"

//---------------------------------------------------------------------------
CGateway::CGateway(SRV_CONTEXT *c) : CThread(), CMonitor()
{
    cntx = c;
    // init
    ss.sin_family = AF_INET;
    ss.sin_port = htons(c->port);
    ss.sin_addr.s_addr = INADDR_ANY;
    srv = ::socket(AF_INET, SOCK_STREAM, 0);
    // start Gateway
    if(srv == INVALID_SOCKET)
    {
        CErrorHandler::Error(SYS_ERROR, ::WSAGetLastError());
    }
    // start server
    else if(::bind(srv, (const sockaddr *)&ss, sizeof(ss)) != 0)
    {
        CErrorHandler::Error(SYS_ERROR, ::WSAGetLastError());
    }
    // listen for connections
    else if(::listen(srv, SOMAXCONN) != 0)
    {
        CErrorHandler::Error(SYS_ERROR, ::WSAGetLastError());
    }
}
//---------------------------------------------------------------------------
CGateway::~CGateway()
{
}
//---------------------------------------------------------------------------
void CGateway::ShutDown()
{
    if(srv != INVALID_SOCKET)
    {
        ::shutdown(srv, SD_BOTH);
        ::closesocket(srv);
    }
    // freee connections
    Lock();
    while(conn.size())
    {
        Kill(*conn.begin());
        conn.erase(conn.begin());
    }
    Unlock();
}
//---------------------------------------------------------------------------
void CGateway::Run()
{
    while(!Terminated(100))
    {
        SOCKET cli = ::accept(srv, NULL, NULL);
        if(cli != INVALID_SOCKET)
        {
            Lock();
            // new connection
            cntx->cli = cli;
            CClient *c = new CClient(cntx);
            conn.push_back(c);
            c->StartThread();
            Unlock();
        }
    }
}
//---------------------------------------------------------------------------
void CGateway::Send(SOCKET socket, uint8_t *hdr, int hdr_size, uint8_t *body, int body_size)
{
    Lock();
    vector<CClient *>::iterator c;
    for(c = conn.begin();c != conn.end();c++)
    {
        CClient *cli = *c;
        if(!cntx->echo && (cli->cli == socket))
        {
            continue;
        }
        if(::send(cli->cli, hdr, hdr_size, 0) == SOCKET_ERROR)
        {
            Kill(cli);
            c = conn.erase(c);
            if(c == conn.end())
            {
                break;
            }
        }
        else if(::send(cli->cli, body, body_size, 0) == SOCKET_ERROR)
        {
            Kill(cli);
            c = conn.erase(c);
            if(c == conn.end())
            {
                break;
            }
        }
    }
    Unlock();
}
//---------------------------------------------------------------------------
void CGateway::Kill(CClient *cli)
{
    cli->Close();
    cli->StopThread();
    cli->WaitFor();
    delete cli;
}
//---------------------------------------------------------------------------

